aboutsummaryrefslogtreecommitdiffstats
path: root/mail/mail-mt.c
diff options
context:
space:
mode:
author5 <NotZed@Ximian.com>2001-10-26 05:21:33 +0800
committerMichael Zucci <zucchi@src.gnome.org>2001-10-26 05:21:33 +0800
commitcab9406217ca92439a86e768be464b9ba72e4ed1 (patch)
treedae0f438fda43229f49cbbb5f7ebc2de58848e08 /mail/mail-mt.c
parent7e6933926a9daaa7dd635effe7715bca43cecd00 (diff)
downloadgsoc2013-evolution-cab9406217ca92439a86e768be464b9ba72e4ed1.tar
gsoc2013-evolution-cab9406217ca92439a86e768be464b9ba72e4ed1.tar.gz
gsoc2013-evolution-cab9406217ca92439a86e768be464b9ba72e4ed1.tar.bz2
gsoc2013-evolution-cab9406217ca92439a86e768be464b9ba72e4ed1.tar.lz
gsoc2013-evolution-cab9406217ca92439a86e768be464b9ba72e4ed1.tar.xz
gsoc2013-evolution-cab9406217ca92439a86e768be464b9ba72e4ed1.tar.zst
gsoc2013-evolution-cab9406217ca92439a86e768be464b9ba72e4ed1.zip
So apparently the uicomp can just 'vanish' while we're using it. Joy. Take
2001-10-25 <NotZed@Ximian.com> * folder-browser-ui.c (fbui_sensitize_timeout): So apparently the uicomp can just 'vanish' while we're using it. Joy. Take care of that case here, fixes #13482. (fbui_sensitise_item): Check here too just for kicks. * mail-folder-cache.c (store_finalised): If we can't destroy our async event, then queue another one to do it. (store_finalised_finish): And handle it here, until we can, then free it. (mail_note_store): Queue an async event to get folderinfo, dont use mail_get_folderinfo. (update_folders_get): thread-async event to retrieve the folderinfo, and build it, then queues gui-async event to update the gui. (add_unmatched_info): Taken from mail-ops, adds unmatched if required. (add_vtrash_info): From mail-ops, add trash if required. (update_folders): Thread async event to update gui. (mail_note_store): Ref the store and storage when created. (update_1folder): Changed to assume we have info_lock, and store updates in an updates list. (setup_folder): Same. (folder_changed): Changed to call update_1folder directly. (real_folder_changed): Removed. (mail_note_folder): Changed to call update_1folder directly. (real_note_folder): Removed. (store_folder_subscribed): Call setup_folder directly. (real_folder_created): Removed. (real_flush_update): Function that actually does the updates in the gui thread. (mail_note_store): Go back to using mail_get_folderinfo. (update_folders): Fixed upf ro changed api's. (unset_folder_info): Changed to queue pending updates. (real_folder_deleted): Removed. (store_folder_unsubscribed): Do the removal work directly. (mail_note_store): Dont link to finalised event of store - we now ref it. (mail_note_store_remove): If we have any pending updates, clear them out. Also cancel any pending folderinfo retrieve operations. (update_folders): Remove our update from the storeinfo list, if it still exists. (update_1folder): Make 'sent folder shows all counts' optional via an environmental variable EVOLUTION_COUNT_SENT for all those bloody whinging lusers out there. (mail_note_store_remove): Unref the storage when done. * mail-mt.c (mail_async_event_emit): If we're in main and have a gui task, set it to run via an idle function. (idle_async_event): Wrapper for calling do_async_event from idle function, and freeing the message when done. (idle_async_event): Call mail_msg_free not free on the finished message. * component-factory.c (mail_remove_storage): Destroy the storage async. (store_disconnect): This does the work. (free_storage): Un-note the store when we remove it, so the store noting code can unref things properly. (idle_quit): Return false when done, dont loop. 2001-10-24 <NotZed@Ximian.com> * component-factory.c (owner_set_cb): Setup an async_event handler. (idle_quit): Try to destroy the async_event, or keep dropping out if it can't (deadlock). * mail-mt.c (do_async_event): Set the threadid of the thread we're running in so we know its running/which thread its in. (mail_async_event_emit): Added new argument 'type' which is the type of thread to execute against, gui or another one. Fixed all callers. (mail_async_event_destroy): Return -1 if this operation will fail (deadlock possibility). If we're in the thread of the task we're going to wait for, then return a failure (since we will deadlock). (mail_async_event_emit): Chagned to use MailAsyncFunc type as the function type, which just takes 3 void args, change args to suit. * mail-folder-cache.c (mail_note_store): Record the pending update events in a pending list. We should really be able to use an async event for this, but that doesn't return to the gui loop when done :-/ (update_folders): Remove from pending update when done. svn path=/trunk/; revision=14101
Diffstat (limited to 'mail/mail-mt.c')
-rw-r--r--mail/mail-mt.c83
1 files changed, 57 insertions, 26 deletions
diff --git a/mail/mail-mt.c b/mail/mail-mt.c
index 9c0c9bb18a..25ff6c632a 100644
--- a/mail/mail-mt.c
+++ b/mail/mail-mt.c
@@ -213,7 +213,7 @@ void mail_msg_free(void *msg)
g_free(m);
if (activity)
- mail_async_event_emit(mail_async_event, destroy_objects, NULL, activity, NULL);
+ mail_async_event_emit(mail_async_event, MAIL_ASYNC_GUI, (MailAsyncFunc)destroy_objects, NULL, activity, NULL);
}
/* hash table of ops->dialogue of active errors */
@@ -566,8 +566,12 @@ static pthread_mutex_t status_lock = PTHREAD_MUTEX_INITIALIZER;
struct _proxy_msg {
struct _mail_msg msg;
MailAsyncEvent *ea;
- CamelObjectEventHookFunc func;
- CamelObject *o;
+ mail_async_event_t type;
+
+ pthread_t thread;
+
+ MailAsyncFunc func;
+ void *o;
void *event_data;
void *data;
};
@@ -577,12 +581,22 @@ do_async_event(struct _mail_msg *mm)
{
struct _proxy_msg *m = (struct _proxy_msg *)mm;
+ m->thread = pthread_self();
m->func(m->o, m->event_data, m->data);
+ m->thread = ~0;
g_mutex_lock(m->ea->lock);
- m->ea->tasks = g_slist_remove(m->ea->tasks, (void *)mm->seq);
+ m->ea->tasks = g_slist_remove(m->ea->tasks, m);
g_mutex_unlock(m->ea->lock);
+}
+static int
+idle_async_event(void *mm)
+{
+ do_async_event(mm);
+ mail_msg_free(mm);
+
+ return FALSE;
}
struct _mail_msg_op async_event_op = {
@@ -602,41 +616,56 @@ MailAsyncEvent *mail_async_event_new(void)
return ea;
}
-int mail_async_event_emit(MailAsyncEvent *ea, CamelObjectEventHookFunc func, CamelObject *o, void *event_data, void *data)
+int mail_async_event_emit(MailAsyncEvent *ea, mail_async_event_t type, MailAsyncFunc func, void *o, void *event_data, void *data)
{
struct _proxy_msg *m;
int id;
int ismain = pthread_self() == mail_gui_thread;
- if (ismain) {
- func(o, event_data, data);
- /* id of -1 is 'always finished' */
- return -1;
- } else {
- /* we dont have a reply port for this, we dont care when/if it gets executed, just queue it */
- m = mail_msg_new(&async_event_op, NULL, sizeof(*m));
- m->func = func;
- m->o = o;
- m->event_data = event_data;
- m->data = data;
- m->ea = ea;
+ /* we dont have a reply port for this, we dont care when/if it gets executed, just queue it */
+ m = mail_msg_new(&async_event_op, NULL, sizeof(*m));
+ m->func = func;
+ m->o = o;
+ m->event_data = event_data;
+ m->data = data;
+ m->ea = ea;
+ m->type = type;
+ m->thread = ~0;
+
+ id = m->msg.seq;
+ g_mutex_lock(ea->lock);
+ ea->tasks = g_slist_prepend(ea->tasks, m);
+ g_mutex_unlock(ea->lock);
- id = m->msg.seq;
- g_mutex_lock(ea->lock);
- ea->tasks = g_slist_prepend(ea->tasks, (void *)id);
- g_mutex_unlock(ea->lock);
- e_msgport_put(mail_gui_port, (EMsg *)m);
- return id;
- }
+ /* We use an idle function instead of our own message port only because the
+ gui message ports's notification buffer might overflow and deadlock us */
+ if (type == MAIL_ASYNC_GUI) {
+ if (ismain)
+ g_idle_add(idle_async_event, m);
+ else
+ e_msgport_put(mail_gui_port, (EMsg *)m);
+ } else
+ e_thread_put(mail_thread_queued, (EMsg *)m);
+
+ return id;
}
-void mail_async_event_destroy(MailAsyncEvent *ea)
+int mail_async_event_destroy(MailAsyncEvent *ea)
{
int id;
+ pthread_t thread = pthread_self();
+ struct _proxy_msg *m;
g_mutex_lock(ea->lock);
while (ea->tasks) {
- id = (int)ea->tasks->data;
+ m = ea->tasks->data;
+ id = m->msg.seq;
+ if (m->thread == thread) {
+ g_warning("Destroying async event from inside an event, returning EDEADLK");
+ g_mutex_unlock(ea->lock);
+ errno = EDEADLK;
+ return -1;
+ }
g_mutex_unlock(ea->lock);
mail_msg_wait(id);
g_mutex_lock(ea->lock);
@@ -645,6 +674,8 @@ void mail_async_event_destroy(MailAsyncEvent *ea)
g_mutex_free(ea->lock);
g_free(ea);
+
+ return 0;
}
/* ********************************************************************** */